@@ -37,11 +37,15 @@ from setuptools import setup as upstream_setup
 from setuptools.dist import Distribution as upstream_Distribution
 
 from . import cmaker
-from .command import (build, build_py, clean,
+from .command import (build, build_ext, build_py, clean,
                       install, install_lib, install_scripts,
                       bdist, bdist_wheel, egg_info,
                       sdist, generate_source_manifest, test)
-from .constants import CMAKE_INSTALL_DIR, CMAKE_SPEC_FILE
+from .constants import (CMAKE_DEFAULT_EXECUTABLE,
+                        CMAKE_INSTALL_DIR,
+                        CMAKE_SPEC_FILE,
+                        set_skbuild_plat_name,
+                        skbuild_plat_name)
 from .exceptions import SKBuildError, SKBuildGeneratorNotFoundError
 from .utils import (mkdir_p, parse_manifestin, PythonModuleFinder, to_platform_path, to_unix_path)
 
@@ -63,6 +67,10 @@ def create_skbuild_argparser():
         '--cmake-executable', default=None, metavar='',
         help='specify the path to the cmake executable'
     )
+    parser.add_argument(
+        '--skip-generator-test', action='store_true',
+        help='skip generator test when a generator is explicitly selected using --generator'
+    )
     return parser
 
 
@@ -70,7 +78,7 @@ def parse_skbuild_args(args, cmake_args, build_tool_args):
     """
     Parse arguments in the scikit-build argument set. Convert specified
     arguments to proper format and append to cmake_args and build_tool_args.
-    Returns the tuple ``(remaining arguments, cmake executable)``.
+    Returns the tuple ``(remaining arguments, cmake executable, skip_generator_test)``.
     """
     parser = create_skbuild_argparser()
     namespace, remaining_args = parser.parse_known_args(args)
@@ -85,13 +93,16 @@ def parse_skbuild_args(args, cmake_args, build_tool_args):
     if namespace.jobs is not None:
         build_tool_args.extend(['-j', str(namespace.jobs)])
 
-    return remaining_args, namespace.cmake_executable
+    if namespace.generator is None and namespace.skip_generator_test is True:
+        sys.exit("ERROR: Specifying --skip-generator-test requires --generator to also be specified.")
+
+    return remaining_args, namespace.cmake_executable, namespace.skip_generator_test
 
 
 def parse_args():
     """This function parses the command-line arguments ``sys.argv`` and returns
-    the tuple ``(setuptools_args, cmake_executable, cmake_args, build_tool_args)`` where each
-    ``*_args`` element corresponds to a set of arguments separated by ``--``."""
+    the tuple ``(setuptools_args, cmake_executable, skip_generator_test, cmake_args, build_tool_args)``
+    where each ``*_args`` element corresponds to a set of arguments separated by ``--``."""
     dutils = []
     cmake = []
     make = []
@@ -110,9 +121,9 @@ def parse_args():
         else:
             argsets[i].append(arg)
 
-    dutils, cmake_executable = parse_skbuild_args(dutils, cmake, make)
+    dutils, cmake_executable, skip_generator_test = parse_skbuild_args(dutils, cmake, make)
 
-    return dutils, cmake_executable, cmake, make
+    return dutils, cmake_executable, skip_generator_test, cmake, make
 
 
 @contextmanager
@@ -132,10 +143,9 @@ def _parse_setuptools_arguments(setup_attrs):
     """This function instantiates a Distribution object and
     parses the command line arguments.
 
-    It returns the tuple
-        ``(display_only, help_commands, commands,
-        hide_listing, force_cmake, skip_cmake, plat_name)``
+    It returns the tuple ``(display_only, help_commands, commands, hide_listing, force_cmake, skip_cmake, plat_name)``
     where
+
     - display_only is a boolean indicating if an argument like '--help',
       '--help-commands' or '--author' was passed.
     - help_commands is a boolean indicating if argument '--help-commands'
@@ -148,6 +158,8 @@ def _parse_setuptools_arguments(setup_attrs):
       explicitly be skipped.
     - plat_name is a string identifying the platform name to embed in generated
       filenames. It defaults to ``distutils.util.get_platform()``.
+    - build_ext_inplace is a boolean indicating if ``build_ext`` command was
+      specified along with the --inplace argument.
 
     Otherwise it raises DistutilsArgError exception if there are
     any error on the command-line, and it raises DistutilsGetoptError
@@ -207,9 +219,11 @@ def _parse_setuptools_arguments(setup_attrs):
             "--plat-name is ambiguous: %s" % ", ".join(plat_names))
     plat_name = list(plat_names)[0]
 
+    build_ext_inplace = dist.get_command_obj('build_ext').inplace
+
     return (display_only, dist.help_commands, dist.commands,
             dist.hide_listing, dist.force_cmake, dist.skip_cmake,
-            plat_name)
+            plat_name, build_ext_inplace)
 
 
 def _check_skbuild_parameters(skbuild_kw):
@@ -286,6 +300,7 @@ def _should_run_cmake(commands, cmake_with_sdist):
     is found in ``commands``."""
     for expected_command in [
         "build",
+        "build_ext",
         "develop",
         "install",
         "install_lib",
@@ -308,18 +323,18 @@ def _save_cmake_spec(args):
     """Save the CMake spec to disk"""
     # We use JSON here because readability is more important than performance
     try:
-        os.makedirs(os.path.dirname(CMAKE_SPEC_FILE))
+        os.makedirs(os.path.dirname(CMAKE_SPEC_FILE()))
     except OSError:
         pass
 
-    with open(CMAKE_SPEC_FILE, 'w+') as fp:
+    with open(CMAKE_SPEC_FILE(), 'w+') as fp:
         json.dump(args, fp)
 
 
 def _load_cmake_spec():
     """Load and return the CMake spec from disk"""
     try:
-        with open(CMAKE_SPEC_FILE) as fp:
+        with open(CMAKE_SPEC_FILE()) as fp:
             return json.load(fp)
     except (OSError, IOError, ValueError):
         return None
@@ -333,16 +348,17 @@ def setup(*args, **kw):  # noqa: C901
 
     The CMake project is re-configured only if needed. This is achieved by (1) retrieving the environment mapping
     associated with the generator set in the ``CMakeCache.txt`` file, (2) saving the CMake configure arguments and
-    version in :const:`skbuild.constants.CMAKE_SPEC_FILE`: and (3) re-configuring only if either the generator or
+    version in :func:`skbuild.constants.CMAKE_SPEC_FILE()`: and (3) re-configuring only if either the generator or
     the CMake specs change.
     """
-    sys.argv, cmake_executable, cmake_args, make_args = parse_args()
+    sys.argv, cmake_executable, skip_generator_test, cmake_args, make_args = parse_args()
 
     # work around https://bugs.python.org/issue1011113
     # (patches provided, but no updates since 2014)
     cmdclass = kw.get('cmdclass', {})
     cmdclass['build'] = cmdclass.get('build', build.build)
     cmdclass['build_py'] = cmdclass.get('build_py', build_py.build_py)
+    cmdclass['build_ext'] = cmdclass.get('build_ext', build_ext.build_ext)
     cmdclass['install'] = cmdclass.get('install', install.install)
     cmdclass['install_lib'] = cmdclass.get('install_lib',
                                            install_lib.install_lib)
@@ -405,7 +421,7 @@ def setup(*args, **kw):  # noqa: C901
     try:
         (display_only, help_commands, commands,
          hide_listing, force_cmake, skip_cmake,
-         plat_name) = \
+         plat_name, build_ext_inplace) = \
             _parse_setuptools_arguments(kw)
     except (DistutilsArgError, DistutilsGetoptError):
         has_invalid_arguments = True
@@ -439,7 +455,7 @@ def setup(*args, **kw):  # noqa: C901
             print('')
         return upstream_setup(*args, **kw)
 
-    developer_mode = "develop" in commands or "test" in commands
+    developer_mode = "develop" in commands or "test" in commands or build_ext_inplace
 
     packages = kw.get('packages', [])
     package_dir = kw.get('package_dir', {})
@@ -456,13 +472,39 @@ def setup(*args, **kw):  # noqa: C901
         for parent_dir, file_list in kw.get('data_files', [])
     }
 
+    # Since CMake arguments provided through the command line have more
+    # weight and when CMake is given multiple times a argument, only the last
+    # one is considered, let's prepend the one provided in the setup call.
+    cmake_args = skbuild_kw['cmake_args'] + cmake_args
+
     if sys.platform == 'darwin':
-
+        # Set plat-name based on CMAKE_OSX_* arguments
         if plat_name is None:
-            # The following code is duplicated in bdist_wheel.finalize_options()
-            plat_name = "macosx-10.6-x86_64"
+            # These default values are duplicated in bdist_wheel.finalize_options()
+            version = '10.6'
+            machine = 'x86_64'
+        else:
+            (_, version, machine) = plat_name.split('-')
 
-        (_, version, machine) = plat_name.split('-')
+        # The loop here allows for CMAKE_OSX_* command line arguments to overload
+        # values passed with either the ``--plat-name`` command-line argument
+        # or the ``cmake_args`` setup option.
+        for cmake_arg in cmake_args:
+            if 'CMAKE_OSX_DEPLOYMENT_TARGET' in cmake_arg:
+                version = cmake_arg.split('=')[1]
+            if 'CMAKE_OSX_ARCHITECTURES' in cmake_arg:
+                machine = cmake_arg.split('=')[1]
+
+        set_skbuild_plat_name("macosx-{}-{}".format(version, machine))
+
+        # Set platform env. variable so that commands (e.g. bdist_wheel)
+        # uses this information. The _PYTHON_HOST_PLATFORM env. variable is
+        # used in distutils.util.get_platform() function.
+        os.environ['_PYTHON_HOST_PLATFORM'] = skbuild_plat_name()
+
+        # Set CMAKE_OSX_DEPLOYMENT_TARGET and CMAKE_OSX_ARCHITECTURES if not already
+        # specified
+        (_, version, machine) = skbuild_plat_name().split('-')
         if not cmaker.has_cmake_cache_arg(
                 cmake_args, 'CMAKE_OSX_DEPLOYMENT_TARGET'):
             cmake_args.append(
@@ -498,17 +540,12 @@ def setup(*args, **kw):  # noqa: C901
             cmake_executable = os.path.join(cmake.CMAKE_BIN_DIR, 'cmake')
             break
 
-    # Since CMake arguments provided through the command line have more
-    # weight and when CMake is given multiple times a argument, only the last
-    # one is considered, let's prepend the one provided in the setup call.
-    cmake_args = skbuild_kw['cmake_args'] + cmake_args
-
     # Languages are used to determine a working generator
     cmake_languages = skbuild_kw['cmake_languages']
 
     try:
         if cmake_executable is None:
-            cmake_executable = 'cmake'
+            cmake_executable = CMAKE_DEFAULT_EXECUTABLE
         cmkr = cmaker.CMaker(cmake_executable)
         if not skip_cmake:
             cmake_minimum_required_version = skbuild_kw['cmake_minimum_required_version']
@@ -517,16 +554,22 @@ def setup(*args, **kw):  # noqa: C901
                     raise SKBuildError(
                         "CMake version %s or higher is required. CMake version %s is being used" % (
                             cmake_minimum_required_version, cmkr.cmake_version))
-            # Used to confirm that the cmake executable is the same
+            # Used to confirm that the cmake executable is the same, and that the environment
+            # didn't change
             cmake_spec = {
-                'args': [which('cmake')] + cmake_args,
-                'version': cmkr.cmake_version
+                'args': [which(CMAKE_DEFAULT_EXECUTABLE)] + cmake_args,
+                'version': cmkr.cmake_version,
+                'environment': {
+                    'PYTHONNOUSERSITE': os.environ.get("PYTHONNOUSERSITE"),
+                    'PYTHONPATH': os.environ.get("PYTHONPATH")
+                }
             }
 
             # skip the configure step for a cached build
             env = cmkr.get_cached_generator_env()
             if env is None or cmake_spec != _load_cmake_spec():
                 env = cmkr.configure(cmake_args,
+                                     skip_generator_test=skip_generator_test,
                                      cmake_source_dir=cmake_source_dir,
                                      cmake_install_dir=skbuild_kw['cmake_install_dir'],
                                      languages=cmake_languages
@@ -542,6 +585,10 @@ def setup(*args, **kw):  # noqa: C901
         print('')
         sys.exit(ex)
 
+    # If any, strip ending slash from each package directory
+    package_dir = {package: prefix[:-1] if prefix[-1] == "/" else prefix
+                   for package, prefix in package_dir.items()}
+
     # If needed, set reasonable defaults for package_dir
     for package in packages:
         if package not in package_dir:
@@ -565,18 +612,20 @@ def setup(*args, **kw):  # noqa: C901
                            py_modules, new_py_modules,
                            scripts, new_scripts,
                            data_files)
+
     if developer_mode:
         # Copy packages
         for package, package_file_list in package_data.items():
             for package_file in package_file_list:
                 package_file = os.path.join(package_dir[package], package_file)
-                cmake_file = os.path.join(CMAKE_INSTALL_DIR, package_file)
+                cmake_file = os.path.join(CMAKE_INSTALL_DIR(), package_file)
                 if os.path.exists(cmake_file):
                     _copy_file(cmake_file, package_file, hide_listing)
+
         # Copy modules
         for py_module in py_modules:
             package_file = py_module + ".py"
-            cmake_file = os.path.join(CMAKE_INSTALL_DIR, package_file)
+            cmake_file = os.path.join(CMAKE_INSTALL_DIR(), package_file)
             if os.path.exists(cmake_file):
                 _copy_file(cmake_file, package_file, hide_listing)
     else:
@@ -587,20 +636,20 @@ def setup(*args, **kw):  # noqa: C901
         _consolidate_package_data_files(original_package_data, package_prefixes, hide_listing)
 
         for data_file in original_manifestin_data_files:
-            dest_data_file = os.path.join(CMAKE_INSTALL_DIR, data_file)
+            dest_data_file = os.path.join(CMAKE_INSTALL_DIR(), data_file)
             _copy_file(data_file, dest_data_file, hide_listing)
 
     kw['package_data'] = package_data
     kw['package_dir'] = {
         package: (
-            os.path.join(CMAKE_INSTALL_DIR, prefix)
-            if os.path.exists(os.path.join(CMAKE_INSTALL_DIR, prefix))
+            os.path.join(CMAKE_INSTALL_DIR(), prefix)
+            if os.path.exists(os.path.join(CMAKE_INSTALL_DIR(), prefix))
             else prefix)
         for prefix, package in package_prefixes
     }
 
     kw['scripts'] = [
-        os.path.join(CMAKE_INSTALL_DIR, script) if mask else script
+        os.path.join(CMAKE_INSTALL_DIR(), script) if mask else script
         for script, mask in new_scripts.items()
     ]
 
@@ -609,6 +658,9 @@ def setup(*args, **kw):  # noqa: C901
         for parent_dir, file_set in data_files.items()
     ]
 
+    if 'zip_safe' not in kw:
+        kw['zip_safe'] = False
+
     # Adapted from espdev/ITKPythonInstaller/setup.py.in
     # pylint: disable=missing-docstring
     class BinaryDistribution(upstream_Distribution):
@@ -636,15 +688,17 @@ def _collect_package_prefixes(package_dir, packages):
     setup call was made with a package list featuring "top" and "top.bar", but
     not "top.not_a_subpackage".
 
-    top/                -> top/
-      __init__.py       -> top/__init__.py                 (parent: top)
-      foo.py            -> top/foo.py                      (parent: top)
-      bar/              -> top/bar/                        (parent: top)
-        __init__.py     -> top/bar/__init__.py             (parent: top.bar)
+    ::
 
-      not_a_subpackage/ -> top/not_a_subpackage/           (parent: top)
-        data_0.txt      -> top/not_a_subpackage/data_0.txt (parent: top)
-        data_1.txt      -> top/not_a_subpackage/data_1.txt (parent: top)
+        top/                -> top/
+          __init__.py       -> top/__init__.py                 (parent: top)
+          foo.py            -> top/foo.py                      (parent: top)
+          bar/              -> top/bar/                        (parent: top)
+            __init__.py     -> top/bar/__init__.py             (parent: top.bar)
+
+          not_a_subpackage/ -> top/not_a_subpackage/           (parent: top)
+            data_0.txt      -> top/not_a_subpackage/data_0.txt (parent: top)
+            data_1.txt      -> top/not_a_subpackage/data_1.txt (parent: top)
 
     The paths in the generated install manifest are matched to packages
     according to the parents indicated on the right.  Only packages that are
@@ -663,7 +717,6 @@ def _collect_package_prefixes(package_dir, packages):
     ))
 
 
-# pylint:disable=too-many-arguments, too-many-branches
 def _classify_installed_files(install_paths, package_data, package_prefixes,
                               py_modules, new_py_modules,
                               scripts, new_scripts,
@@ -672,13 +725,11 @@ def _classify_installed_files(install_paths, package_data, package_prefixes,
     assert not os.path.isabs(cmake_source_dir)
     assert cmake_source_dir != "."
 
-    cmake_source_dir = to_unix_path(cmake_source_dir)
-
-    install_root = os.path.join(os.getcwd(), CMAKE_INSTALL_DIR)
+    install_root = os.path.join(os.getcwd(), CMAKE_INSTALL_DIR())
     for path in install_paths:
         # if this installed file is not within the project root, complain and
         # exit
-        if not to_platform_path(path).startswith(CMAKE_INSTALL_DIR):
+        if not to_platform_path(path).startswith(CMAKE_INSTALL_DIR()):
             raise SKBuildError((
                 "\n  CMake-installed files must be within the project root.\n"
                 "    Project Root  : {}\n"
@@ -686,20 +737,7 @@ def _classify_installed_files(install_paths, package_data, package_prefixes,
                     install_root, to_platform_path(path)))
 
         # peel off the 'skbuild' prefix
-        path = to_unix_path(os.path.relpath(path, CMAKE_INSTALL_DIR))
-
-        # If the CMake project lives in a sub-directory (e.g src), its
-        # include rules are relative to it. If the project is not already
-        # installed in a directory, we need to prepend
-        # the source directory so that the remaining of the logic
-        # can successfully check if the path belongs to a package or
-        # if it is a module.
-        # TODO(jc) Instead of blindly checking if cmake_install_dir is set
-        #          or not, a more elaborated check should be done.
-        if (not cmake_install_dir
-                and cmake_source_dir
-                and not path.startswith(cmake_source_dir)):
-            path = to_unix_path(os.path.join(cmake_source_dir, path))
+        path = to_unix_path(os.path.relpath(path, CMAKE_INSTALL_DIR()))
 
         _classify_file(path, package_data, package_prefixes,
                        py_modules, new_py_modules,
@@ -767,7 +805,7 @@ def _classify_file(path, package_data, package_prefixes,
     if file_set is None:
         file_set = set()
         data_files[parent_dir] = file_set
-    file_set.add(os.path.join(CMAKE_INSTALL_DIR, path))
+    file_set.add(os.path.join(CMAKE_INSTALL_DIR(), path))
 
 
 def _copy_file(src_file, dest_file, hide_listing=True):
@@ -800,7 +838,7 @@ def _consolidate_package_modules(
     both the source tree and the CMake install tree into one location.
 
     The one location is the CMake install tree
-    (see data::`.constants.CMAKE_INSTALL_DIR`).
+    (see :func:`.constants.CMAKE_INSTALL_DIR()`).
 
     Why ? This is a necessary evil because ``Setuptools`` keeps track of
     packages and modules files to install using a dictionary of lists where
@@ -815,7 +853,7 @@ def _consolidate_package_modules(
     one are either already included or missing from the distribution.
 
     Once a module has been identified as ``missing``, it is both copied
-    into the data::`.constants.CMAKE_INSTALL_DIR` and added to the
+    into the :func:`.constants.CMAKE_INSTALL_DIR()` and added to the
     ``package_data`` dictionary so that it can be considered by
     the upstream setup function.
     """
@@ -825,7 +863,7 @@ def _consolidate_package_modules(
         # and cmake install tree.
         modules = PythonModuleFinder(
             packages, package_dir, py_modules,
-            alternative_build_base=CMAKE_INSTALL_DIR
+            alternative_build_base=CMAKE_INSTALL_DIR()
         ).find_all_modules()
     except DistutilsError as msg:
         raise SystemExit("error: {}".format(str(msg)))
@@ -842,7 +880,7 @@ def _consolidate_package_modules(
 
         # Copy missing module file
         if os.path.exists(src_module_file):
-            dest_module_file = os.path.join(CMAKE_INSTALL_DIR, src_module_file)
+            dest_module_file = os.path.join(CMAKE_INSTALL_DIR(), src_module_file)
             _copy_file(src_module_file, dest_module_file, hide_listing)
 
         # Since the mapping in package_data expects the package to be associated
@@ -867,7 +905,7 @@ def _consolidate_package_modules(
 
 def _consolidate_package_data_files(original_package_data, package_prefixes, hide_listing):
     """This function copies package data files specified using the ``package_data`` keyword
-    into data::`.constants.CMAKE_INSTALL_DIR`.
+    into :func:`.constants.CMAKE_INSTALL_DIR()`.
 
     ::
 
@@ -880,7 +918,7 @@ def _consolidate_package_data_files(original_package_data, package_prefixes, hid
     Considering that (1) the packages associated with modules located in both the source tree and
     the CMake install tree are consolidated into the CMake install tree, and (2) the consolidated
     package path set in the ``package_dir`` dictionary and later used by setuptools to package
-    (or install) modules and data files is data::`.constants.CMAKE_INSTALL_DIR`, copying the data files
+    (or install) modules and data files is :func:`.constants.CMAKE_INSTALL_DIR()`, copying the data files
     is required to ensure setuptools can find them when it uses the package directory.
     """
     project_root = os.getcwd()
@@ -893,5 +931,5 @@ def _consolidate_package_data_files(original_package_data, package_prefixes, hid
             for src_data_file in glob(expanded_package_dir):
                 full_prefix_length = len(os.path.join(project_root, prefix)) + 1
                 data_file = src_data_file[full_prefix_length:]
-                dest_data_file = os.path.join(CMAKE_INSTALL_DIR, prefix, data_file)
+                dest_data_file = os.path.join(CMAKE_INSTALL_DIR(), prefix, data_file)
                 _copy_file(src_data_file, dest_data_file, hide_listing)
