@@ -7,18 +7,23 @@ import os.path
 import pkg_resources
 import py.path
 import re
+import requests
 import six
 import subprocess
 import sys
-
+import wheel
 
 from contextlib import contextmanager
-
 from mock import patch
+from pkg_resources import parse_version
+
+from skbuild import __version__ as skbuild_version
 from skbuild.compat import which  # noqa: F401
 from skbuild.utils import push_dir
 from skbuild.platform_specifics import get_platform
 
+from zipfile import ZipFile
+
 
 SAMPLES_DIR = os.path.join(
     os.path.dirname(os.path.realpath(__file__)),
@@ -68,7 +73,7 @@ def _tmpdir(basename):
     the pytest settings."""
 
     # Adapted from _pytest.tmpdir.tmpdir()
-    basename = re.sub("[\W]", "_", basename)
+    basename = re.sub(r"[\W]", "_", basename)
     max_val = 30
     if len(basename) > max_val:
         basename = basename[:max_val]
@@ -196,6 +201,11 @@ def execute_setup_py(project_dir, setup_args, disable_languages_test=False):
     # See https://stackoverflow.com/questions/9160227/dir-util-copy-tree-fails-after-shutil-rmtree
     distutils.dir_util._path_created = {}
 
+    # Clear _PYTHON_HOST_PLATFORM to ensure value sets in skbuild.setuptools_wrap.setup() does not
+    # influence other tests.
+    if '_PYTHON_HOST_PLATFORM' in os.environ:
+        del os.environ['_PYTHON_HOST_PLATFORM']
+
     with push_dir(str(project_dir)), push_argv(["setup.py"] + setup_args), prepend_sys_path([str(project_dir)]):
 
         # Restore master working set that is reset following call to "python setup.py test"
@@ -273,3 +283,57 @@ def get_cmakecache_variables(cmakecache):
                 variable_value = line.split("=")[1]
                 results[variable_name] = (variable_type, variable_value)
     return results
+
+
+def is_site_reachable(url):
+    """Return True if the given website can be accessed"""
+    try:
+        request = requests.get(url)
+        return request.status_code == 200
+    except requests.exceptions.ConnectionError:
+        return False
+
+
+def check_wheel_content(wheel_archive, expected_distribution_name, expected_content, pure=False):
+    """This function raises an AssertionError if the given wheel_archive
+    does not have the expected content.
+
+    Note that this function already takes care of appending the
+    ``<expected_distribution_name>.dist-info`` files to the ``expected_content``
+    list.
+    """
+
+    expected_content = list(expected_content)
+    expected_content += [
+        '%s.dist-info/top_level.txt' % expected_distribution_name,
+        '%s.dist-info/WHEEL' % expected_distribution_name,
+        '%s.dist-info/RECORD' % expected_distribution_name,
+        '%s.dist-info/METADATA' % expected_distribution_name
+    ]
+
+    if parse_version(wheel.__version__) < parse_version('0.31.0'):
+        expected_content += [
+            '%s.dist-info/DESCRIPTION.rst' % expected_distribution_name,
+            '%s.dist-info/metadata.json' % expected_distribution_name
+        ]
+
+    if pure:
+        assert wheel_archive.endswith('-none-any.whl')
+    else:
+        assert not wheel_archive.endswith('-none-any.whl')
+
+    archive = ZipFile(wheel_archive)
+    member_list = archive.namelist()
+
+    assert sorted(expected_content) == sorted(member_list)
+
+    # PEP-0427: Generator is the name and optionally the version of the
+    # software that produced the archive.
+    # See https://www.python.org/dev/peps/pep-0427/#file-contents
+    current_generator = None
+    with archive.open("%s.dist-info/WHEEL" % expected_distribution_name) as wheel_file:
+        for line in wheel_file:
+            if line.startswith(b"Generator"):
+                current_generator = line.split(b":")[1].strip()
+                break
+    assert current_generator == six.b("skbuild %s" % skbuild_version)
